@@ -54,7 +54,7 @@ import gitrepo, hgrepo, util
 from git_handler import GitHandler
 import verify
 
-testedwith = '2.8.2 3.0.1 3.1'
+testedwith = '2.8.2 3.0.1 3.1 3.2.2 3.3'
 buglink = 'https://bitbucket.org/durin42/hg-git/issues'
 
 cmdtable = {}
@@ -219,15 +219,28 @@ extensions.wrapfunction(hg, 'peer', peer)
 @util.transform_notgit
 def exchangepull(orig, repo, remote, heads=None, force=False, bookmarks=()):
     if isinstance(remote, gitrepo.gitrepo):
+        # transaction manager is present in Mercurial >= 3.3
+        try:
+            trmanager = getattr(exchange, 'transactionmanager')
+        except AttributeError:
+            trmanager = None
         pullop = exchange.pulloperation(repo, remote, heads, force,
                                         bookmarks=bookmarks)
+        if trmanager:
+            pullop.trmanager = trmanager(repo, 'pull', remote.url())
         lock = repo.lock()
         try:
             pullop.cgresult = repo.githandler.fetch(remote.path, heads)
-            pullop.closetransaction()
+            if trmanager:
+                pullop.trmanager.close()
+            else:
+                pullop.closetransaction()
             return pullop
         finally:
-            pullop.releasetransaction()
+            if trmanager:
+                pullop.trmanager.release()
+            else:
+                pullop.releasetransaction()
             lock.release()
     else:
         return orig(repo, remote, heads, force, bookmarks=bookmarks)
