@@ -20,12 +20,12 @@ class overlaymanifest(object):
         self.repo = repo
         self.tree = repo.handler.git.get_object(sha)
         self._map = None
-        self._flagmap = None
+        self._flags = None
 
     def withflags(self):
         self.load()
-        return set([path for path, flag in self._flagmap.iteritems()
-                    if flag & 020100])
+        return set([path for path, flag in self._flags.iteritems()
+                    if flag != ''])
 
     def copy(self):
         return overlaymanifest(self.repo, self.tree.id)
@@ -40,6 +40,15 @@ class overlaymanifest(object):
     def flags(self, path):
         self.load()
 
+        return hgflag(self._flags[path])
+
+    def load(self):
+        if self._map is not None:
+            return
+
+        self._map = {}
+        self._flags = {}
+
         def hgflag(gitflag):
             if gitflag & 0100:
                 return 'x'
@@ -48,15 +57,6 @@ class overlaymanifest(object):
             else:
                 return ''
 
-        return hgflag(self._flagmap[path])
-
-    def load(self):
-        if self._map is not None:
-            return
-
-        self._map = {}
-        self._flagmap = {}
-
         def addtree(tree, dirname):
             for entry in tree.iteritems():
                 if entry.mode & 040000:
@@ -66,10 +66,21 @@ class overlaymanifest(object):
                 else:
                     path = dirname + entry.path
                     self._map[path] = bin(entry.sha)
-                    self._flagmap[path] = entry.mode
+                    self._flags[path] = hgflag(entry.mode)
 
         addtree(self.tree, '')
 
+    def matches(self, match):
+        '''generate a new manifest filtered by the match argument'''
+        if match.always():
+            return self.copy()
+
+        mf = self.copy()
+        for fn in mf.keys():
+            if not match(fn):
+                del mf[fn]
+        return mf
+
     def iteritems(self):
         self.load()
         return self._map.iteritems()
@@ -82,6 +93,40 @@ class overlaymanifest(object):
         self.load()
         return self._map[path]
 
+    def __contains__(self, path):
+        self.load()
+        return path in self._map
+
+    def get(self, path, default=None):
+        self.load()
+        return self._map.get(path, default)
+
+    def diff(self, m2, clean=False):
+        self.load()
+        if isinstance(m2, overlaymanifest):
+            m2.load()
+
+        # below code copied from manifest.py:manifestdict.diff
+        diff = {}
+
+        for fn, n1 in self.iteritems():
+            fl1 = self._flags.get(fn, '')
+            n2 = m2.get(fn, None)
+            fl2 = m2._flags.get(fn, '')
+            if n2 is None:
+                fl2 = ''
+            if n1 != n2 or fl1 != fl2:
+                diff[fn] = ((n1, fl1), (n2, fl2))
+            elif clean:
+                diff[fn] = None
+
+        for fn, n2 in m2.iteritems():
+            if fn not in self:
+                fl2 = m2._flags.get(fn, '')
+                diff[fn] = ((None, ''), (n2, fl2))
+
+        return diff
+
     def __delitem__(self, path):
         del self._map[path]
 
@@ -180,6 +225,10 @@ class overlaychangectx(context.changectx):
         except ImportError:
             return 1
 
+    def totuple(self):
+        return (self.commit.tree, self.user(), self.date(), self.files(),
+                self.description(), self.extra())
+
 class overlayrevlog(object):
     def __init__(self, repo, base):
         self.repo = repo
@@ -250,8 +299,10 @@ class overlaychangelog(overlayrevlog):
             sha = self.node(sha)
         if sha == nullid:
             return (nullid, "", (0, 0), [], "", {})
-        return overlaychangectx(self.repo, sha)
-
+        try:
+            return self.base.read(sha)
+        except LookupError:
+            return overlaychangectx(self.repo, sha).totuple()
 
 class overlayrepo(object):
     def __init__(self, handler, commits, refs):
@@ -273,6 +324,13 @@ class overlayrepo(object):
 
         self._makemaps(commits, refs)
 
+        try:
+            # Mercurial >= 3.3
+            from mercurial import namespaces
+            self.names = namespaces.namespaces()
+        except ImportError:
+            pass
+
     def __getitem__(self, n):
         if n not in self.revmap:
             return self.handler.repo[n]
